# frozen_string_literal: true

# Generated via
#  `rails generate hyrax:work CrcDataset`
class CrcDataset < ActiveFedora::Base
  include ::Hyrax::WorkBehavior
  include ::Hyrax::TombstoneBehavior
  include ExternalServices
  include S3FileHandleable
  self.indexer = CrcDatasetIndexer
  # Change this to restrict which works can be added as a child.
  # self.valid_child_concerns = []
  validates :title, presence: { message: 'Your CRC dataset must have a title.' }
  validate :validate_parent_collection

  before_save :set_format_and_size
  after_save :save_metadata_as_json_in_s3, :save_meta_json_in_s3
  after_create :set_default_source_and_tombstone_status, :set_default_values, :register_ark
  after_destroy :clear_s3_and_all_associated_objects


  CRC_FILTER_FACET_FIELDS = [
    "year_recorded_sim",
    "complex_subject_species_sim",
    "complex_subject_type_sim",
    "complex_subject_sex_sim",
    "complex_subject_age_itsim",
    "session_sim",
    "modality_sim"
  ]

  CRC_FACET_FIELDS = [
    "complex_identifier_group_id_sim",
    "coverage_sim",
    "crc_resource_type_sim"
  ] + CRC_FILTER_FACET_FIELDS

  # ------ properties from core metadata ------
  # property date_modified          - not displayed (filled in by the system)
  # property date_uploaded          - not displayed (filled in by the system)
  # property depositor              - not displayed (filled in by the system)
  # property title                  - keep

  # ------ properties from basic metadata ------
  # property alternative_title      - keep
  # property label                  - not displayed, used for file version label
  # property relative_path          - not displayed, used for file storage
  # property import_url             - not displayed, used for file imports
  # property resource_type          - keep
  # property creator                - do not display
  # property contributor            - do not display
  # property description            - keep
  # property abstract               - keep
  # property keyword                - keep
  # property license                - keep
  # property rights_notes           - do not display
  # property rights_statement       - do not display
  # property access_right           - do not display
  # property publisher              - keep
  # property date_created           - do not display
  # property subject                - keep
  # property language               - keep
  # property identifier             - do not display (used to store alternate identifiers)
  # property based_near             - do not display
  # property related_url            - do not display
  # property bibliographic_citation - not displayed (generated by the system)
  # property source                 - do not display (should be filled in by the system for bulk imports)

  # ------ properties from Datacite metadata ------
  property :doi, predicate: ::RDF::Vocab::DataCite.doi, multiple: false do |index|
    index.as :symbol
  end

  property :tombstone_status, predicate: ::RDF::Vocab::Rdms.tombstoneStatus, multiple: false do |index|
    index.as :symbol
  end

  property :tombstone_date, predicate: ::RDF::Vocab::Rdms.tombstoneDate, multiple: false do |index|
    index.as :symbol
  end

  property :is_tombstoned, predicate: ::RDF::Vocab::Rdms.isTombstone, multiple: false do |index|
    index.as :symbol
  end

  property :complex_person, predicate: ::RDF::Vocab::SIOC.has_creator, class_name: 'ComplexPerson'

  # property :date_published, predicate: ::RDF::Vocab::Rdms.datePublished, multiple: false do |index|
  #   index.type :date
  #   index.as :stored_sortable
  # end

  property :complex_date, predicate: ::RDF::Vocab::DC.date, class_name: 'ComplexDate'

  property :complex_identifier, predicate: ::RDF::Vocab::Rdms.identifier, class_name: 'ComplexIdentifier'

  # size and format to be obtained from the files attached
  property :format, predicate: ::RDF::Vocab::Rdms.format do |index|
    index.as :symbol
  end

  property :files_size, predicate: ::RDF::Vocab::Rdms.size do |index|
    index.type :integer
    index.as :stored_searchable
  end

  # description and abstract is in the basic metadata. Ignoring other descriptions for now.

  # Geolocation - would this be used? Ignoring for now.

  property :complex_funding_reference, predicate: ::RDF::Vocab::DataCite.fundref, class_name: 'ComplexFundingReference'

  property :complex_relation, predicate: ::RDF::Vocab::DC.relation, class_name: 'ComplexRelation'

  # ------ properties from CRC metadata ------

  property :parent_work_id, predicate: ::RDF::Vocab::Rdms.parentWorkId, multiple: false do |index|
    index.as :symbol
  end

  property :crc_work_type, predicate: ::RDF::Vocab::Rdms.workType, multiple: false do |index|
    index.as :stored_searchable, :facetable
  end

  property :crc_resource_type, predicate: ::RDF::Vocab::Rdms.crcResourceType, multiple: false do |index|
    index.as :stored_searchable, :facetable
  end

  property :modality, predicate: ::RDF::Vocab::Rdms.modality do |index|
    index.as :stored_searchable, :facetable
  end

  property :folder_name, predicate: ::RDF::Vocab::Rdms.folderName do |index|
    index.as :stored_searchable
  end

  property :experiment_description, predicate: ::RDF::Vocab::Rdms.experimentDescription do |index|
    index.as :stored_searchable
  end

  property :approval_number, predicate: ::RDF::Vocab::Rdms.approvalNumber, multiple: false do |index|
    index.as :symbol
  end

  property :extra_information, predicate: ::RDF::Vocab::Rdms.extraInformation do |index|
    index.as :stored_searchable
  end

  property :software_version, predicate: ::RDF::Vocab::Rdms.softwareVersion do |index|
    index.as :stored_searchable
  end

  property :coverage, predicate: ::RDF::Vocab::DC.coverage, multiple: false do |index|
    index.as :stored_searchable, :facetable
  end

  # Required due to bug saving nested resources
  property :updated_subresources, predicate: ::RDF::Vocab::Rdms.updatedSubresources, class_name: "ActiveTriples::Resource"

  # This must be included at the end, because it finalizes the metadata
  # schema (by adding accepts_nested_attributes)
  include ::Hyrax::BasicMetadata
  include ComplexValidation
  accepts_nested_attributes_for :complex_person, reject_if: :person_blank, allow_destroy: true
  accepts_nested_attributes_for :complex_date, reject_if: :date_blank, allow_destroy: true
  accepts_nested_attributes_for :complex_identifier, reject_if: :identifier_blank, allow_destroy: true
  accepts_nested_attributes_for :complex_funding_reference, reject_if: :fundref_blank, allow_destroy: true
  accepts_nested_attributes_for :complex_relation, reject_if: :relation_blank, allow_destroy: true
  accepts_nested_attributes_for :updated_subresources, allow_destroy: true

  def complex_subjects
    ComplexSubject.where(parent_source_identifier: source.first)
  end

  def complex_sessions
    complex_subjects.map(&:complex_sessions)
  end

  def complex_modalities
    complex_subjects.map(&:complex_modalities)
  end

  def file_sets_fast_load_metadata
    FileSetFastLoadMetadata.sort_and_filter_by_title_and_source(source.first)
  end

  def default_admin_set
    AdminSet.where(title: ENV.fetch('CRC_ADMIN_SET_TITLE', 'CRC 1280 Publishing Workflow')).first
  end

  def set_default_funding_reference
    funding_reference = ComplexFundingReference.new(::RDF::Node.new, ActiveTriples::Resource.new)
    funding_reference.funder_identifier = ENV['CRCDATASET_FUNDER_IDENTIFIER']
    funding_reference.funder_name = ENV['CRCDATASET_FUNDER_NAME']
    funding_reference.award_title = ENV['CRCDATASET_AWARD_TITLE']
    funding_reference.award_number = ENV['CRCDATASET_AWARD_NUMBER']
    funding_reference.award_uri = ENV['CRCDATASET_AWARD_URI']
    self.complex_funding_reference = self.complex_funding_reference + [funding_reference]
  end

  private

  def set_default_source_and_tombstone_status
    self.source = source.present? ? source : [SecureRandom.uuid]
    self.is_tombstoned = false
    self.save
  end

  def validate_parent_collection
    self.parent_collections.each do |parent_collection|
      if parent_collection.present? && parent_collection.collection_type.title != 'CRC 1280'
        self.errors.add(:base, "The CRC 1280 works can only be added to CRC 1280 collection")
      end
    end
  end

  def set_default_values
    self.language = [ENV['CRCDATASET_LANGUAGE']] unless self.language.present?
    self.resource_type = [ENV['CRCDATASET_RESOURCE_TYPE']] unless self.resource_type.present?
    self.save
  end

end
